"use strict";
this.name           = "AstroLibrary";
this.author         = "Stranger";
this.copyright      = "Creative Commons: attribution, non-commercial, sharealike with clauses - see readme.txt";
this.description    = "Library of oostronomical functions";
this.version        = "4.3";

// SunEngine functions

this.$astroLib_sunSpectrum = function(sunRadius)    // sun spectrum class as function of sun radius
    {
    if (system.isInterstellarSpace) return;
    var spectrum;
    var spectrumSelector;
    if (sunRadius <= 600000)
        {
        spectrumSelector = Math.floor(sunRadius / 30000);
        }
    else
        {
        if(sunRadius > 1000000)
            {
            spectrumSelector = Math.floor((sunRadius + 500000) / 15000);
            }
        else
            {
            spectrumSelector = Math.floor(sunRadius / 10000);
            }
        }
    switch (spectrumSelector)
        {
        // M class red dwarf
        case 12: spectrum = "M3.5V"; break;
        case 13: spectrum = "M3V"; break;
        case 14: spectrum = "M2.5V"; break;
        case 15: spectrum = "M2V"; break;
        case 16: spectrum = "M1.5V"; break;
        case 17: spectrum = "M1V"; break;
        case 18: spectrum = "M0.5V"; break;
        case 19: spectrum = "M0V"; break;
        // K class orange dwarf
        case 60: spectrum = "K9.5V"; break;
        case 61: spectrum = "K9V"; break;
        case 62: spectrum = "K8.5V"; break;
        case 63: spectrum = "K8V"; break;
        case 64: spectrum = "K7.5V"; break;
        case 65: spectrum = "K7V"; break;
        case 66: spectrum = "K6.5V"; break;
        case 67: spectrum = "K6V"; break;
        case 68: spectrum = "K5.5V"; break;
        case 69: spectrum = "K5V"; break;
        case 70: spectrum = "K4.5V"; break;
        case 71: spectrum = "K4V"; break;
        case 72: spectrum = "K3.5V"; break;
        case 73: spectrum = "K3V"; break;
        case 74: spectrum = "K2.5V"; break;
        case 75: spectrum = "K2V"; break;
        case 76: spectrum = "K1.5V"; break;
        case 77: spectrum = "K1V"; break;
        case 78: spectrum = "K0.5V"; break;
        case 79: spectrum = "K0V"; break;
        // G class yellow dwarf
        case 80: spectrum = "G9.5V"; break;
        case 81: spectrum = "G9V"; break;
        case 82: spectrum = "G8.5V"; break;
        case 83: spectrum = "G8V"; break;
        case 84: spectrum = "G7.5V"; break;
        case 85: spectrum = "G7V"; break;
        case 86: spectrum = "G6.5V"; break;
        case 87: spectrum = "G6V"; break;        
        case 88: spectrum = "G5.5V"; break;
        case 89: spectrum = "G5V"; break;
        case 90: spectrum = "G4.5V"; break;
        case 91: spectrum = "G4V"; break;
        case 92: spectrum = "G3.5V"; break;
        case 93: spectrum = "G3V"; break;
        case 94: spectrum = "G2.5V"; break;
        case 95: spectrum = "G2V"; break;
        case 96: spectrum = "G1.5V"; break;
        case 97: spectrum = "G1V"; break;
        case 98: spectrum = "G0.5V"; break;
        case 99: spectrum = "G0V"; break;
        // F class white hot star
        case 100: spectrum = "F9.5V"; break;
        case 101: spectrum = "F9V"; break;
        case 102: spectrum = "F8.5V"; break;
        case 103: spectrum = "F8V"; break;
        case 104: spectrum = "F7.5V"; break;
        case 105: spectrum = "F7V"; break;
        case 106: spectrum = "F6.5V"; break;
        case 107: spectrum = "F6V"; break;
        default: spectrum = null;
        };    
    return spectrum;
    }

this.$astroLib_sunTemperature = function(sunRadius)    // sun temperature as function of sun radius
    {
    if (system.isInterstellarSpace) return;
    var solarTemperature;
    if (sunRadius <= 600000)
        {
        solarTemperature = 4000 - (600000 - sunRadius) / 300;
        }
    else
        {
        if (sunRadius > 1000000)
            {
            solarTemperature = 6000 + (sunRadius - 1000000) / 300;
            }
        else
            {
            solarTemperature = 6000 - (1000000 - sunRadius) / 200;
            }
        }  
    var solarTemperatureTweaked = solarTemperature - 25 + 50 * system.scrambledPseudoRandomNumber(119);
    return solarTemperatureTweaked;
    }

this.$astroLib_sunLuminosity = function(sunTemperature, sunRadius)    // sun luminosity as function of sun radius
    {
    if (system.isInterstellarSpace) return;   
    var sunLuminosity = Math.pow((sunTemperature / 6000),4) * Math.pow((sunRadius / 1000000),2); // Stefan-Boltzmann law
    return sunLuminosity;
    }

this.$astroLib_sunMass = function(sunRadius)    // sun mass as function of sun radius
    {
    if (system.isInterstellarSpace) return;
    var solarMass;
    if (sunRadius <= 300000)
        {
    solarMass = 0.175;
        }
    else
        {
        solarMass = 1.0 - (960000 - sunRadius) / 1000000 * 1.25;
        }    
    var sunMassTweaked = solarMass * (0.975 + 0.05 * system.scrambledPseudoRandomNumber(120));
    return sunMassTweaked;
    }

this.$astroLib_solPeriod = function(sunMass)    // main planet period as function of sun mass
    {
    if (system.isInterstellarSpace) return;   
    var standardOU = 63750 * 675;   // Sol-Terra system twin AU
    var standardYear = 365.25;   // standard year for Sol-Terra system twin
    var mainOrbitVector = new Vector3D(system.sun.position.subtract(system.mainPlanet.position));
    var ouScale = mainOrbitVector.magnitude();  // OU in custom system
    var mainPeriod = standardYear * Math.pow((ouScale / standardOU),1.5) / Math.pow(sunMass,0.5); // 3rd Kepler low
    return mainPeriod;
    }

this.$astroLib_orbitRadius = function()         // planet orbits in OU
    {
    if (system.isInterstellarSpace) return;
    var mainOrbitVector = new Vector3D(system.sun.position.subtract(system.mainPlanet.position));
    var ouScale = mainOrbitVector.magnitude();
    var planetsOrbitsOU = new Array();
    var planetOrbitVector;
    for (let i=0; i<system.planets.length; i++)
        {
        planetOrbitVector = new Vector3D(system.sun.position.subtract(system.planets[i].position));
        planetsOrbitsOU[i] = planetOrbitVector.magnitude() / ouScale;
        }
    return planetsOrbitsOU;    
    }

this.$astroLib_mainPlanetTemperature = function(sunLuminosity,sunDistance,mainPlanetRadius)  // main planet temperature - simplified function of radius
    {
    if (system.isInterstellarSpace) return;    
    var mainInsolation = sunLuminosity * Math.pow((50000000 / sunDistance),2);
    var equilibriumTemperature = 255 * Math.pow(mainInsolation,0.25);
    var mainTemperature;
    mainTemperature = equilibriumTemperature + (mainPlanetRadius - 4800) * 3 / 400 + 20;    
    return mainTemperature;
    }

this.$astroLib_SolarWind = function(sunDistance)    // solar wind. Used to collect fuel
	{
    if (system.isInterstellarSpace) return;
    var solarWindDensity = (system.sun.radius * system.sun.radius) / (sunDistance * sunDistance) * system.info.corona_flare;
    if (player.alertCondition != 1)
        {
        solarWindDensity = 0;
        }
    var solarWindFlux;
    if(player.ship.speed > player.ship.maxSpeed)
        {
        solarWindFlux = solarWindDensity * player.ship.maxSpeed;
        }
    else
        {
        solarWindFlux = solarWindDensity * player.ship.speed;      //rate of fuel collection, LY/min          
        }
    return solarWindFlux;
	}

// PlanetEngine functions

this.$astroLib_insolation = function(sunLuminosity, sunDistance)    // insolation level
    {
    if (system.isInterstellarSpace) return;   
    var insolation = sunLuminosity * Math.pow((50000000 / sunDistance),2);
    return insolation;    
    }

this.$astroLib_equilibriumTemperature = function(insolation)    // equilibrium temperature without greenhouse effect. Albedo same as Earth
    {
    if (system.isInterstellarSpace) return;   
    var equilibriumTemperature = 255 * Math.pow(insolation,0.25);
    return equilibriumTemperature;
    }

this.$astroLib_planetDensity = function(planetRadius, equilibriumTemperature)   // planet density as function of planet radius and equilibrium temperature
    {
    if (system.isInterstellarSpace) return; 
    var planetDensity = 0.925 + Math.sqrt(planetRadius / 302.5) * Math.sqrt(equilibriumTemperature / 255);
    return planetDensity;
    }

this.$astroLib_escapeVelocity = function(planetRadius, planetDensity)   // escape velocity as function of planet radius and density
    {
    if (system.isInterstellarSpace) return;
    var escapeVelocity = 11180 * planetRadius / 6400 * Math.sqrt(planetDensity / 5.5);
    return escapeVelocity;
    }

this.$astroLib_surfaceGravity = function(planetRadius, escapeVelocity)  // planet surface gravity as function of planet radius and escape velocity
    {
    if (system.isInterstellarSpace) return;
    var surfaceGravity = escapeVelocity * escapeVelocity / (planetRadius * 1000) / (9.81 * 2);
    return surfaceGravity;
    }

this.$astroLib_atmosphereMass = function(planetRadius, escapeVelocity, equilibriumTemperature) // mass of atmosphere per unit of surface square as function of planet radius, escape velocity and equilibrium temperature
    {
    if (system.isInterstellarSpace) return;
    var termosphereTemperature = equilibriumTemperature * 4;
    var termosphereVelocity = 320 * Math.sqrt(termosphereTemperature / 273.15);
    var dissipation_ratio = termosphereVelocity / escapeVelocity;
    var dissipation_adjust = dissipation_ratio * 25;
    var volatile_pool = planetRadius / 1600 * Math.pow(255 / equilibriumTemperature,2);
    var atmosphereMass;
    var insolation = Math.pow(equilibriumTemperature / 255,4);
    var volatile_adjust = (planetRadius - 6250) / 100;
    if (insolation > 1.75 && planetRadius > 6250 && planetRadius < 7500)    // runaway greenhouse trigger
        {
        atmosphereMass = volatile_pool * Math.exp(-dissipation_adjust) * (1 + volatile_adjust * volatile_adjust);
        }
    else
        {
        atmosphereMass = volatile_pool * Math.exp(-dissipation_adjust);
        }
    return atmosphereMass;
    }

this.$astroLib_atmospherePressure = function(atmosphereMass, surfaceGravity)    // atmosphere pressure as function of atmosphere mass and surface gravity
    {
    if (system.isInterstellarSpace) return;
    var atmospherePressure = atmosphereMass * surfaceGravity;
    return atmospherePressure;
    }

this.$astroLib_atmosphereTemperature = function(equilibriumTemperature, atmosphereMass) // planet surface temperature as function of equilibrium temperature and atmosphere mass (greenhouse effect)
    {
    if (system.isInterstellarSpace) return;
    var atmosphereTemperature = equilibriumTemperature + 0.125 * equilibriumTemperature * Math.sqrt(atmosphereMass);
    return atmosphereTemperature;
    }
