"use strict";
this.name    = "SolarSystemData";
this.author = "Stranger";
this.copyright = "This work is hereby placed in the public domain.";
this.description = "Displays system sun and planet data on MFD and F4 info page.";
this.version = "4.5.2";

this.startUpComplete = function()
    {
    this.self = player.ship;
    this.$setupDelay = new Timer(this, this.$setupSystemInfo, 1);    //delay to set planet orbits!
    }

this.shipWillLaunchFromStation = this.shipWillExitWitchspace = function()
    {
    if (system.isInterstellarSpace) return;
    if(this.self.equipmentStatus("EQ_ADVANCED_COMPASS") == "EQUIPMENT_OK")
        {
        this.self.awardEquipment("EQ_ADVANCED_SYSTEM_DATA_MFD");
        this.$scriptDelay = new Timer(this, this.$setupSystemInfo, 1);
        this.$startUpdateMFDTimer();
        }
    else
        {
        this.self.removeEquipment("EQ_ADVANCED_SYSTEM_DATA_MFD");
        }
    }

this.$startUpdateMFDTimer = function()
	{
	if(this.$updateMFDTimer)
		{ this.$updateMFDTimer.start(); }
	else
		{ this.$updateMFDTimer = new Timer(this, this.$updateMFD,0,1); }
	}

this.shipWillDockWithStation = this.shipDied = this.$stop$updateMFDTimer = function()
	{   
	if(this.$updateMFDTimer && this.$updateMFDTimer.isRunning) { this.$updateMFDTimer.stop(); } 
	}

this.$updateMFD = function()
    {
    if(this.self.equipmentStatus("EQ_ADVANCED_COMPASS") != "EQUIPMENT_OK") return;
    if(this.self.compassMode === "COMPASS_MODE_SUN")
        {
        this.$starPage();
        }
    else
        {
        this.$planetPage();
        }
    }

this.$starPage = function()
	{
    var w = worldScripts.AstroLibrary;
	var info = system.info;
	var catNum = " " + galaxyNumber + " " + system.ID;
    if (system.info.sun_radius)
		{
        var sunRadius = system.sun.radius;
		}
    var sunDistance = player.ship.position.distanceTo(system.sun).toFixed(0);
    var sunRelDistance = (sunDistance/system.sun.radius);
    var solarWind = w.$astroLib_SolarWind(sunDistance);
    var solarActivity = 10 * info.corona_flare;

    var output = "Catalog number: " + catNum + "\n"
	                + "Spectral class: " + this.$spectrumText + "\n"
	                + "Radius: " + this.$solRadius.toFixed(2) + "\n"
	                + "Mass: " + this.$sunMass.toFixed(2) + "\n"
                    + "Luminosity: " + this.$solLuminosity.toFixed(2) + "\n"
	                + "Solar activity: " + solarActivity.toFixed(2) + "\n"                    
                    + "Main planet period: " + this.$localYear.toFixed(2) + " days" + "\n"
	                + "Sun distance: " + sunRelDistance.toFixed(2) + " R" + "\n"
	                + "Solar wind flux: " + solarWind.toFixed(2) + " LY/min" + "\n\n";
    
    this.self.setMultiFunctionText("EQ_ADVANCED_SYSTEM_DATA_MFD", output, false);
	}

this.$planetPage = function()
    {
    var output = this.$planetOrbitInfo;
    this.self.setMultiFunctionText("EQ_ADVANCED_SYSTEM_DATA_MFD", output, false);
    }

this.$setupSystemInfo = function()
    {
    var w = worldScripts.AstroLibrary;
    // sun info
	var info = system.info;
	var catNum = " " + galaxyNumber + " " + system.ID;
    if (system.info.sun_radius)
		{
        var sunRadius = system.sun.radius;
		}
    var spectrumText = w.$astroLib_sunSpectrum(sunRadius);
    var solRadius = system.sun.radius/950000 * (0.99 + 0.02 * system.scrambledPseudoRandomNumber(119));
    var sunMass = w.$astroLib_sunMass(sunRadius);
    var sunTemperature = w.$astroLib_sunTemperature(sunRadius);
    var sunLuminosity = w.$astroLib_sunLuminosity(sunTemperature,sunRadius);
    var solLuminosity = 1.31 * sunLuminosity; // use for display only, not in real calculations!
    var localYear = w.$astroLib_solPeriod(sunMass);
    this.$spectrumText = spectrumText;
    this.$solRadius = solRadius;
    this.$sunMass = sunMass;
    this.$solLuminosity = solLuminosity;
    this.$localYear = localYear;
    // planet info
    var infoFlag = 1;
    var planetMaxNumber = 8;
    var planetOrbitInfoRow = new Array();
    planetOrbitInfoRow[0] = "1.00 OU * Main Planet\n";
    for (let i=1; i<planetMaxNumber; i++)
        {
        planetOrbitInfoRow[i] = "";
        } 
    var planetOrbitRadius;
    var planetRadius;
    var planetDataTag;
    var planetOrbitScan = w.$astroLib_orbitRadius();
    var planetOrbitInfo = "PLANETS" + "\n";
    for (let i=0; i<system.planets.length; i++)
        {
        planetOrbitRadius = planetOrbitScan[i];
        planetRadius = system.planets[i].radius;

        if(system.planets[i].radius < 25000)     
            {
            planetDataTag = 0;
            }
        if(system.planets[i].radius >= 27500 && system.planets[i].radius < 30000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 1;
            } 
        if(system.planets[i].radius >= 50000 && system.planets[i].radius < 75000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 2;
            }
        if(system.planets[i].radius >= 30000 && system.planets[i].radius < 50000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 3;
            }
        if(system.planets[i].radius >= 75000 && system.planets[i].radius < 100000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 4;
            }
        if(system.planets[i].radius >= 150000 && system.planets[i].radius < 175000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 5;
            } 
        if(system.planets[i].radius >= 125000 && system.planets[i].radius < 150000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 6;
            } 
        if(system.planets[i].radius >= 100000 && system.planets[i].radius < 125000 && system.planets[i] != system.mainPlanet)
            {
            planetDataTag = 7;
            } 
        if(system.planets[i].radius >= 25000 && system.planets[i].radius < 27500 && system.planets[i] != system.mainPlanet)       
            {
            planetDataTag = 8;
            }
        
        switch (planetDataTag)
            {
            case 0: infoFlag = 0; break;
            case 1: planetOrbitInfoRow[1] = planetOrbitRadius.toFixed(2) + " OU * Miniterra\n"; break;
            case 2: planetOrbitInfoRow[2] = planetOrbitRadius.toFixed(2) + " OU * Terra\n"; break;
            case 3: planetOrbitInfoRow[3] = planetOrbitRadius.toFixed(2) + " OU * Subterra\n"; break;
            case 4: planetOrbitInfoRow[4] = planetOrbitRadius.toFixed(2) + " OU * Superterra\n"; break;
            case 5: planetOrbitInfoRow[5] = planetOrbitRadius.toFixed(2) + " OU * Gas Giant\n"; break;
            case 6: planetOrbitInfoRow[6] = planetOrbitRadius.toFixed(2) + " OU * Gas Giant\n"; break;
            case 7: planetOrbitInfoRow[7] = planetOrbitRadius.toFixed(2) + " OU * Ice Giant\n"; break;
            case 8: planetOrbitInfoRow[8] = planetOrbitRadius.toFixed(2) + " OU * Dwarf Terra\n"; break;
            }
        }

    for (let i=0; i<planetOrbitInfoRow.length; i++)
        {
        planetOrbitInfo += planetOrbitInfoRow[i];
        }
        this.$planetOrbitInfo = planetOrbitInfo;
    
    if(this.self.docked) this.$initInterface(this.self.dockedStation);
    }

this.shipWillDockWithStation = this.shipWillEnterWitchspace = function()
	{
    this.shipDied();
    }

this.shipDied = function()
	{
	if(this.$scriptDelay) 
		{
		this.$scriptDelay.stop();
		delete this.$scriptDelay;
		}
    }

this.shipDockedWithStation = function(station)
    {
    this.$setupSystemInfo();    
    if (this.self.dockedStation.isMainStation == true)
        {
        this.$initInterface(station);    
        }
    }

this.$initInterface = function(station)
    {
	station.setInterface(this.name,{
		title:"System Data Sheet",
		category:"Informational",
		summary:"Provide advanced system data",
		callback:this.$setupSystemInfoPage.bind(this)
        });
    }

this.$setupSystemInfoPage = function()
    {
    var systemAdvancedInfo = "SUN" + "\n"
	                + "Spectral class: " + this.$spectrumText + "\n"
	                + "Radius: " + this.$solRadius.toFixed(2) + "\n"
	                + "Mass: " + this.$sunMass.toFixed(2) + "\n"
                    + "Luminosity: " + this.$solLuminosity.toFixed(2) + "\n"                   
                    + "Main planet period: " + this.$localYear.toFixed(2) + " days" + "\n"
                    + "\n"
                    + this.$planetOrbitInfo;

    mission.runScreen({
        title: "System Data Sheet",
//        background: {name: "specialCargoLoad.png", height: 480 },
        message: systemAdvancedInfo,}
        );     
    }